# 树形控件
>完整展现数据层级关系，并具有展开、收起、选择等交互功能。树形控件需要在插件中构造数据。

## 控件对象
`kd.bos.form.control.TreeView`

## 属性说明
属性名 | 属性ID | 类型 | 默认值 | 说明 
---- | :----: | :----: | :----: | ---- 
根节点可见 | `rootVisible` | boolean | `true` | 设置根节点是否可见
多选 | `multi` | boolean | `false` | 设置节点是否允许多选
选择父节点 | `cpn` | boolean | `false` | 设置子节点取消选中是否影响父级节点
启用拖拽 | `dragEnable` | boolean | `false` | 设置节点可拖拽
级联全选 | `choseChildNode` | boolean | `true` | 设置勾选上级节点时，下级节点全选
大数据优化 | `bigDataRender` | boolean | `true` | 数据量过大进行优化处理
手风琴模式 | `accordion` | boolean | `false` | 勾选后，目录树同一级只能展开一个控制

?> 控件通用属性[请参考](advanced/controlProperties/)

## 插件代码范例

- **基本使用，插件中构造节点数据**

![default.png](./image/default.png)

```java
public class TestBillPlugin extends AbstractFormPlugin {

  @Override
  public void afterCreateNewData(EventObject e) {
    TreeView tv1 = this.getView().getControl("树控件标识");
    this.constructorData(tv1);
  }

  /**
   * 构造数据
   */
  private void constructorData(TreeView tv1) {
    final String rootId = "0"; // 根节点id
    TreeNode rootNode = new TreeNode(null, rootId, "根节点", true);
    rootNode.setIsOpened(true); // 设置默认展开

    for (int i = 0; i < 2; i++) {
      // 构造一级子节点
      String p1 = rootId + "-" + i;
      TreeNode tn1 = new TreeNode(rootId, p1, p1, true);
      tn1.setIsOpened(true);
      tn1.setColor("red"); // 设置节点文字颜色
      for (int j = 0; j < 2; j++) {
        // 构造二级子节点
        String p2 = p1 + "-" + j;
        TreeNode tn2 = new TreeNode(p1, p2, p2, false);
        tn1.addChild(tn2);
      }
      rootNode.addChild(tn1);
    }
    tv1.addNode(rootNode);
  }
``` 

- **异步加载数据**

> 首次点击展开节点，异步加载下级节点数据，需要插件实现`queryTreeNodeChildren`方法

![asyncload.gif](./image/asyncload.gif)
```java
// 第一步：插件实现树节点查询接口
public class TreeviewPlugin extends AbstractFormPlugin implements  TreeNodeQueryListener {

  @Override
  public void registerListener(EventObject e) {
    super.registerListener(e);

    TreeView tv1 = this.getView().getControl("树控件标识");
    tv1.addTreeNodeQueryListener(this);
  }

  @Override
  public void afterCreateNewData(EventObject e) {
    super.afterCreateNewData(e);

    TreeView tv = this.getView().getControl("树控件标识");
    constructorData(tv);
  }
  
  private void constructorData(TreeView tv) {
    final String rootId = "1"; // 根节点id
    TreeNode rootNode = new TreeNode(null, rootId, "根节点", true);
    rootNode.setIsOpened(true); // 设置默认展开

    for (int i = 0; i < 2; i++) {
      // 构造一级子节点
      String p1 = rootId + "-" + i;
      TreeNode tn1 = new TreeNode(rootId, p1, "Expand to load", true);
      rootNode.addChild(tn1);
    }
    tv.addNode(rootNode);
  }

  @Override
  public void queryTreeNodeChildren(TreeNodeEvent event) {
  	// 响应查询事件
    String id = (String) event.getNodeId();
    TreeView tv = (TreeView) event.getSource();
    List<TreeNode> nodes = new ArrayList<>();
    if ("1-0".equals(id)) {
      nodes.add(new TreeNode(id, "1-0-0", "深圳"));
      nodes.add(new TreeNode(id, "1-0-1", "香港"));
      tv.addNodes(nodes);
    } else if ("1-1".equals(id)) {
      nodes.add(new TreeNode(id, "1-1-0", "广州"));
      nodes.add(new TreeNode(id, "1-1-1", "珠海"));
      tv.addNodes(nodes);
    }
  }
}
```  


- **节点选择 treeNodeCheck 事件**

事件触发时机: 树控件支持多选，用户点击节点前的复选框时，触发此事件。

```java
TreeView treeView = this.getView().getControl(KEY_TREEVIEW1);
treeView.setMulti(true);	// 支持多选
```

插件可以在此事件，即时获取当前选择的节点。
说明：必须在界面加载时，设置树形控件，支持多选

```java
package kd.bos.plugin.sample.dynamicform.pcform.control.template;

import java.util.EventObject;
import java.util.List;

import kd.bos.dataentity.utils.StringUtils;
import kd.bos.form.control.TreeView;
import kd.bos.form.control.events.TreeNodeCheckEvent;
import kd.bos.form.control.events.TreeNodeCheckListener;
import kd.bos.form.plugin.AbstractFormPlugin;

public class TreeViewTreeNodeCheck extends AbstractFormPlugin implements TreeNodeCheckListener {
	
	private final static String KEY_TREEVIEW1 = "treeviewap1";

	@Override
	public void registerListener(EventObject e) {
		super.registerListener(e);
		
		// 侦听树节点勾选事件
		TreeView treeView = this.getView().getControl(KEY_TREEVIEW1);
		treeView.addTreeNodeCheckListener(this);
	}

	@Override
	public void beforeBindData(EventObject e) {
		super.beforeBindData(e);
		TreeView treeView = this.getView().getControl(KEY_TREEVIEW1);
		treeView.setMulti(true);	// 支持多选
	}

	@Override
	public void treeNodeCheck(TreeNodeCheckEvent arg0) {
		TreeView treeView = (TreeView) arg0.getSource();
		if (StringUtils.equals(treeView.getKey(), KEY_TREEVIEW1)){
			List<String> selectNodeIds = treeView.getTreeState().getCheckedNodeIds();
			// TODO 在此添加业务逻辑
		}
	}
}
```

- **节点拖拽**

> 需要打开`启用拖拽`属性
> 插件中需要实现`treeNodeDragged`方法

![dragnode.gif](./image/dragnode.gif)
```java
// 第一步：插件实现树节点选择、拖拽接口
public class TreeviewPlugin extends AbstractFormPlugin 
    implements TreeNodeCheckListener, TreeNodeDragListener {

  @Override
  public void registerListener(EventObject e) {
    super.registerListener(e);

    TreeView tv2 = this.getView().getControl("树控件标识");
    tv2.addTreeNodeDragListener(this);
  }

  @Override
  public void afterCreateNewData(EventObject e) {
    super.afterCreateNewData(e);

    TreeView tv3 = this.getView().getControl("树控件标识");
    constructTV3Data(tv3);
  }

  private void constructTV3Data(TreeView tv3) {
    final String rootId = "0"; // 根节点id
    TreeNode rootNode = new TreeNode(null, rootId, "根节点", true);
    rootNode.setIsOpened(true); // 设置默认展开

    for (int i = 0; i < 2; i++) {
      // 构造一级子节点
      String p1 = rootId + "-" + i;
      TreeNode tn1 = new TreeNode(rootId, p1, p1, true);
      tn1.setIsOpened(true);
      for (int j = 0; j < 2; j++) {
        // 构造二级子节点
        String p2 = p1 + "-" + j;
        TreeNode tn2 = new TreeNode(p1, p2, p2, false);
        tn1.addChild(tn2);
      }
      rootNode.addChild(tn1);
    }
    tv3.addNode(rootNode);
    // 默认选中节点
    tv3.checkNode(new TreeNode(rootId, "0-0", "0-0", true));
    this.getPageCache().put(tv3.getKey(), SerializationUtils.toJsonString(rootNode));
  }

  @Override
  public void treeNodeDragged(TreeNodeDragEvent event) {
    // 拖拽节点
    TreeView tv = (TreeView) event.getSource();
    TreeNode rootNode = SerializationUtils.fromJsonString(this.getPageCache().get(tv.getKey()), TreeNode.class);
    String nodeId = (String) event.getNodeId();
    String fromParentId = (String) event.getFromParentId();
    String toParentId = (String) event.getToParentId();
    if (nodeId.equals(toParentId) || fromParentId.equals(toParentId)) {
      return;
    }
    if (rootNode != null) {
      // 当前拖拽节点
      TreeNode currentNode = rootNode.getTreeNode(nodeId, 16);
      currentNode.setParentid(toParentId);
      
      // 目标节点
      TreeNode targetNode = currentNode.getTreeNode(toParentId, 16);
      if (targetNode != null) {
        this.getView().showTipNotification("父节点不能拖拽到子节点下");
        return;
      }
      
      TreeNode sourceParentNode = rootNode.deleteChildNode(nodeId);
      List<TreeNode> childNodes = sourceParentNode.getChildren();
      if (childNodes != null && childNodes.isEmpty()) {
        // 判断拖拽后是否还存在子节点，如果不存在将children设置null
        sourceParentNode.setChildren(null);
      }

      targetNode = rootNode.getTreeNode(toParentId, 16);
      targetNode.addChild(currentNode);
    }
    tv.updateNode(rootNode);
    this.getPageCache().put(tv.getKey(), SerializationUtils.toJsonString(rootNode));
  }

  @Override
  public void treeNodeCheck(TreeNodeCheckEvent event) {
    // 节点选中或未选中响应方法
    // do something
  }
}
```  

### API
- TreeView

方法名 | 参数 | 说明  
------- | ------- | -------  
`expend` | `String` 节点id | 设置节点展开
`showNode` | `String` 节点id | 展开节点包括其上级节点  
`collapse` | `String` 节点id | 折叠节点 
`deleteNode` | `String` 节点id| 删除一个节点
`deleteNodes` | `List<String>` 一组节点id | 删除一组节点
`deleteAllNodes` |无| 删除所有节点
`updateNode` | `TreeNode` 节点对象| 更新一个节点
`updateNodes` | `List<TreeNode>` 一组节点对象| 更新一组节点
`addNode` | `TreeNode` 节点对象| 增加一个节点
`addNodes` | `List<TreeNode>` 一组节点对象| 增加一组节点
`checkNode` | `TreeNode` 节点对象| 选中一个节点
`checkNodes`|`List<TreeNode>` 一组节点对象| 选中一组节点
`uncheckNode`| `String` 节点id| 取消勾选指定节点，包含子节点
`uncheckNodes`|`List<String>` 一组节点id|取消勾选一组节点，包含子节点
`uncheckNodeWithoutChild` | `String` 节点id | 取消勾选节点，不包括子节点
`uncheckNodesWithoutChild` | `List<String>` 一组节点id | 取消勾选一组节点，不包括子节点
`fouceNode`|`TreeNode` 节点对象| 聚焦到一个节点
`setRootVisiable`|`boolean`| 设置根节点是否可见，`true`可见，`false`不可见
`setMulti`|`boolean`|设置节点是否可以多选

- TreeNode

方法名 | 参数 |返回值 | 说明  
------- | -------|----- | -------  
`TreeNode` | `String, String, String` 父节点id，节点id，节点名称| `TreeNode`| 构造方法
`TreeNode` | `String, String, String, Object` 父节点id，节点id，节点名称，附加数据| `TreeNode`| 构造方法
`TreeNode` | `String, String, String, boolean` 父节点id，节点id，节点名称，是否父节点|`TreeNode` | 构造方法
`TreeNode` | `String, String, String, boolean，Object` 父节点id，节点id，节点名称，是否父节点，附加数据|`TreeNode` | 构造方法
`setId` | `String` 节点id| | 设置节点id  
`getId` ||`String`|获取节点id
`setText` | `String` 节点显示内容| | 设置节点显示内容 
`getText`||`String`|获取节点显示内容
`setParentid` | `String` 父节点id| | 设置父节点id
`getParentid` | |`String`| 获取父节点id
`setChildren` | `List<TreeNode>` 一组节点对象|| 添加一组子节点
`getChildren` ||`List<TreeNode>`| 获取所有子节点
`setData` | `Object` 附加数据|| 设置节点附加数据
`getData`||`Object`| 获取节点附加数据
`setExpend`|`boolean`是否展开||设置节点是否展开
`isExpend`|| `boolean`| 节点是否展开
`setColor`|`String` 颜色|| 设置节点文字颜色
`getColor`|| `String`| 获取节点颜色
`setLongNumber`|`String` 长编码||设置节点长编码
`getLongNumber`||`String`|获取节点长编码
`addChild`|`TreeNode` 节点对象| `TreeNode`| 添加单个子节点
`addChild`|`int, TreeNode` 索引，节点对象| `TreeNode`| 特定位置添加单个子节点
`addChildren` | `List<TreeNode>` 一组节点对象|`TreeNode`| 添加一组子节点
`getTreeNode`|`String, int` 节点id，最大遍历深度| `TreeNode`| 给定节点id查找节点
`getTreeNodeListById`|`List<TreeNode>, String, int` 一组节点，节点id，最大遍历深度| `List<TreeNode>`| 根据id模糊查找所有匹配节点
`getTreeNodeByText`|`String, int` 节点显示内容，最大遍历深度，| `TreeNode`| 根据节点内容查找节点
`getTreeNodeListByText`|`List<TreeNode>, String, int` 一组节点，节点显示内容，最大遍历深度| `List<TreeNode>`| 根据节点内容模糊查找所有匹配节点
`getNodeLevel`|`String, int` 节点id，开始层级|`int`节点层级|获取节点层级，没有找到返回`-1`
`getChildPath`|`String`节点id|`List<String>`子节点路径|获取子节点路径
`deleteChildNode`|`String`节点id|`TreeNode`|递归删除子节点
`Parse`|`String, Map<String, Object>, String, String, String` 父节点id， 数据对象，节点id，节点内容，子节点数据|`TreeNode`|将`Map<String, Object>`对象转换成`TreeNode`